#!/bin/sh
###########################################################BeginFile############
# Copyright:  (c) 2006 by Consul Risk Management B.V.  All Rights Reserved
#
# Name:
#   getoradblogs2.sh
#
# Purpose:
#   Back-end generic actuator script that collects database records from certain
#   Oracle-database table using Oracle's utility sqlplus.
#
# History:
#   20080417 ScienceSoft-SA  EE0704007:  Created. 
#   20080429 ScienceSoft-SA  EE0704007: Fix work with empty TABLE_HEAD. 
#   20080430 ScienceSoft-SA  EE0704007: Add default PARAM_OUTPUT values. Processing headerlines. Remove first character from headerline. 
#   20080503 ScienceSoft-SA  EE0704007: Additional information is wrote to log. 
#   20080508 ScienceSoft-SA  EE0704007: Fix incorrect work on Sun Solaris OS. 
#   20080520 ScienceSoft-SA  EE0704007: Change method of generation of headerline. New variables and abilities. Cosmetic changes.
#   20080527 ScienceSoft-SA  EE0704007: Fix empty oracle user password issue.
#   20080626 ScienceSoft-SA  EE0704007: Fix empty values of env. variables at log when 9 and 10 versions are used. 
#   20080630 ScienceSoft-SA  EE0704007: Fix headline issue. Fix call of rm ${PROPSFILE}0
#   20080805 ScienceSoft-SA  EE0704007: Fix headline truncation on AIX.  Fix processing of TABLE_HEAD9 and TABLE_HEAD10.
#   20080902 ScienceSoft-SA  EE0704007: Fix initial timestamp determination in gobackintime() function.
#
###########################################################EndFile##############

######################################################################
# Store state information.                                                        #
######################################################################
store_state_information()
{
  if [ -s "$PROPSFILE" ]  
  then
    mv "$PROPSFILE" "${PROPSFILE}0"
    grep -v "$TIMESTAMPNAME" "${PROPSFILE}0" >"$PROPSFILE"
    rm -f "${PROPSFILE}0"
  fi
    
  if [ ! -z "${LAST_TIME_NOW}" ]
  then
    echo "$TIMESTAMPNAME=${LAST_TIME_NOW}">>"$PROPSFILE"
  else
    if [ ! -z "${LAST_TIME_BEFORE}" ]
    then
      echo "$TIMESTAMPNAME=${LAST_TIME_BEFORE}">>"$PROPSFILE"
    else
      debug "Nothing to save in Properties File"
    fi
  fi
}

######################################################################
# function returns the date ago
######################################################################
gobackintime()
{
CURRENT_YEAR=`date +%Y`
CURRENT_MONTH=`date +%m`
CURRENT_DAY=`date +%d`
CURRENT_TIME=`date +"%H:%M:%S"`

# checking for leap year
LEAP=0

if [ `expr $CURRENT_YEAR % 400` -eq 0 ] ; then
 LEAP=1
else
 if [ `expr $CURRENT_YEAR % 100` -eq 0 ] ; then 
   LEAP=0
 else
  if [ `expr $CURRENT_YEAR % 4` -eq 0 ] ; then
   LEAP=1
  fi  
 fi 
fi

NEW_DAY=`expr $CURRENT_DAY - 2`
NEW_MONTH=$CURRENT_MONTH
NEW_YEAR=$CURRENT_YEAR

if [ $NEW_DAY -lt 1 ] ; then
case $CURRENT_MONTH in
 1 | "01")
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=12
   NEW_YEAR=`expr $CURRENT_YEAR - 1`
   ;;
 2 | "02")
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 3 | "03")
   if [ $LEAP -eq 1 ] ; then
    NEW_DAY=`expr $NEW_DAY + 29`
    NEW_MONTH=`expr $CURRENT_MONTH - 1`
   else
    NEW_DAY=`expr $NEW_DAY + 28`
    NEW_MONTH=`expr $CURRENT_MONTH - 1`
   fi
   ;;
 4 | "04")
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 5 | "05")
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 6 | "06")
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 7 | "07")
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 8 | "08")
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 9 | "09")
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 10)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 11)
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 12)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
   *) our_exit 1 "Error checking month number, process finished with error."
   ;;
esac
fi

# adding "0" character if number contains only one digit
NEW_MONTH=`echo $NEW_MONTH | sed "s/^[1-9]$/0\$NEW_MONTH/"`
NEW_DAY=`echo $NEW_DAY | sed "s/^[1-9]$/0\$NEW_DAY/"`
debug "Initial collect time: $NEW_YEAR-$NEW_MONTH-$NEW_DAY $CURRENT_TIME"
echo "$NEW_YEAR-$NEW_MONTH-$NEW_DAY $CURRENT_TIME"
}

#######################################################
# Usage: our_exit <exit-code> [exit-message]
#######################################################
our_exit()
{
  debug $2
  finish_log_in_error $1
  exit 0
}

finish_log_in_error()
{
  rm -f "$TEMPFILE"
  echo "#Fields: ">"$TEMPFILE"
  echo "CeA/ExtendIT Debug Log finished by $SCRIPTNAME on `hostname` (`uname`), `date`. Error=$1" >> $CEA_GENACT_DEBUG_LOG
}

test_sqlplus_code()
{
  ret_val=$?
  if [ ${ret_val} -ne 0 ]
  then
    debug "sqlplus returned(${ret_val}): $SQLPLUS_RETURN"
    our_exit 2 "Error occured logging into Oracle via Sqlplus !"
  fi

  retval=`echo "$SQLPLUS_RETURN" | grep -c "ORA-"`
  if [ $retval -eq 1 ]
  then
    # sqlplus return an ORA-xxx error
    our_exit 2 "Error, sqlplus returned: $SQLPLUS_RETURN"
  fi
}

export ORACLE_HOME 
export ORACLE_SID

# Source our function library. It contains generic functions.
. $CEA_GENACT_FUNCLIB || exit 1

####  log received parameters getoradblogs2.sh ##########################################################################
debug "getoradblogs2.sh received following parameters: "
debug "  CEA_GENACT_FUNCLIB = $CEA_GENACT_FUNCLIB"
debug "  CEA_GENACT_DEBUG_LOG = $CEA_GENACT_DEBUG_LOG"
debug "  PROPSFILE = ${PROPSFILE}"
debug "  ORACLE_SID = $ORACLE_SID"
debug "  ORA_INSTANCE = $ORA_INSTANCE"
debug "  ORA_USER = $ORA_USER"
if [ ! -z "${ORA_PWD}" ]
then
  debug "  NON-empty Oracle user password"
fi
debug "  ORACLE_HOME = $ORACLE_HOME"
debug "  TABLE_NAME = $TABLE_NAME"
debug "  TABLE_DESC = $TABLE_DESC"
debug "  TABLE_HEAD = $TABLE_HEAD"
debug "  COL_TIME = $COL_TIME"
debug "  LINE_SIZE = $LINE_SIZE"
debug "  GETHEADER = $GETHEADER"
debug "  NOTIMESTAMP  =  ${NOTIMESTAMP}"
if [ -z "${PARAM_OUTPUT}" ]
then
  debug "  PARAM_OUTPUT is empty"
else
  debug "  NON-empty PARAM_OUTPUT"
fi
debug "  TIMESTAMPNAME = $TIMESTAMPNAME"
debug "  NOLINESCHECK = $NOLINESCHECK"


if [ -z "${TEMPFILE}" ]; then our_exit 1 "TEMPFILE must be specified!"; fi
if [ -z "${ORACLE_SID}" ]; then debug "Warning: 'Oracle Instance' must be specified!"; fi
if [ -z "${ORA_USER}" ]; then our_exit 1 "Option 'Oracle User Name' must be specified!"; fi
if [ -z "${ORACLE_HOME}" ]; then our_exit 1 "Option 'Oracle Home Path' must be specified!"; fi
if [ -z "${PROPSFILE}" ]; then our_exit 1 "Properties file must be specified!"; fi

SPOOLFILE="${TEMPFILE}.lst"

## you can use following param_strings to get right dateformat : 
# alter session set nls_timestamp_format = 'YYYY-MM-DD HH24:MI:SS';
# alter session set nls_date_format = 'YYYY-MM-DD HH24:MI:SS';
if [ -z "${PARAM_OUTPUT}" ]
then
  if [ ! -z "${GETHEADER}" ]
  then
    PARAM_OUTPUT="SET COLSEP ';';
    SET HEA ON;
    SET HEADSEP ON;
    SET HEADS ';';"
  else
    PARAM_OUTPUT="SET COLSEP ';';"
  fi
fi

if [ -z "$TIMESTAMPNAME" ]
then
  TIMESTAMPNAME="LAST_TIME_BEFORE"
fi

######################################################################
# Check if ORACLE_HOME/bin/sqlplus is existing.                      #
######################################################################
debug "Check if ${ORACLE_HOME}/bin/sqlplus is existing..."
if [ ! -s "${ORACLE_HOME}/bin/sqlplus" ]
then
  our_exit 2 "${ORACLE_HOME}/bin/sqlplus not found."
else
  debug "${ORACLE_HOME}/bin/sqlplus (OK)."
fi


######################################################################
# Check major Oracle database release number.                        #
######################################################################
debug "Check major Oracle database release number..."
SQLPLUS_RETURN=`${ORACLE_HOME}/bin/sqlplus -S ${ORA_USER}/${ORA_PWD}${ORA_INSTANCE} << ! 2>>$CEA_GENACT_DEBUG_LOG
set feedback off;
set echo off;
set head off;
set pagesize 0;
select VALUE
from NLS_DATABASE_PARAMETERS
where PARAMETER = 'NLS_RDBMS_VERSION';
exit;
!`
test_sqlplus_code

RDBMS_VERSION=`echo ${SQLPLUS_RETURN} | sed -e "s/\(.*\)\..*\..*\..*\..*/\1/"`
RDBMS_VERSION_LENGTH=`echo "$RDBMS_VERSION"|wc -m`
if [ ${RDBMS_VERSION_LENGTH} -gt 3 ]
then
  our_exit 2 "Obtain major Oracle database release number finished with error. (NOK)."
fi
if [ ${RDBMS_VERSION} -gt 8 ]
then
  debug "Major Oracle database release number: ${RDBMS_VERSION} (OK)."
  if [ ${RDBMS_VERSION} -gt 10 ]
  then
    ${RDBMS_VERSION}='10'
  fi
else
  our_exit 2 "Unsupported major Oracle database release number: ${RDBMS_VERSION} (NOK)."
fi


######################################################################
# Obtain timestamp value from properties file.                       #
######################################################################
if [ -s "${PROPSFILE}" ]
then
  LAST_TIME_BEFORE=`grep -w $TIMESTAMPNAME $PROPSFILE | sed -e "s/$TIMESTAMPNAME=//"`
else
  debug "${PROPSFILE} is empty or not exist."
fi

######################################################################
# Obtain specific table name, table description, timestamp column.   #
######################################################################

if [ -z "${TABLE_HEAD}" ]; then eval "TABLE_HEAD=\${TABLE_HEAD${RDBMS_VERSION}}"; debug "  TABLE_HEAD = $TABLE_HEAD"; fi

if [ -z "${GETHEADER}" ]
then
  if [ -z "${TABLE_HEAD}" ]
  then
    PARAM_HEADER="set hea off;
    set pagesize 0;
    spool ${SPOOLFILE};"
  else
    PARAM_HEADER="set hea off;
    set pagesize 0;
    spool ${SPOOLFILE};
    select replace(${TABLE_HEAD},CHR(10),'') from dual;"
  fi
else
  PARAM_HEADER="spool ${SPOOLFILE};"
fi

if [ -z "${TABLE_NAME}" ]; then eval "TABLE_NAME=\${TABLE_NAME${RDBMS_VERSION}}"; debug "  TABLE_NAME = $TABLE_NAME"; fi
if [ -z "${TABLE_NAME}" ]; then our_exit 1 "Missing table name."; fi

if [ -z "${TABLE_DESC}" ]; then eval "TABLE_DESC=\${TABLE_DESC${RDBMS_VERSION}}"; debug "  TABLE_DESC = $TABLE_DESC"; fi
if [ -z "${TABLE_DESC}" ]; then our_exit 1 "Missing table description."; fi

if [ -z "${NOTIMESTAMP}" ]
then
  if [ -z "${COL_TIME}" ]; then eval "COL_TIME=\${COL_TIME${RDBMS_VERSION}}"; debug "  COL_TIME = $COL_TIME"; fi
  if [ -z "${COL_TIME}" ]; then our_exit 1 "Missing timestamp column."; fi
fi

if [ -z "${LINE_SIZE}" ]; then LINE_SIZE="20000"; debug "  LINE_SIZE = $LINE_SIZE"; fi

LAST_TIME_NOW=""
if [ -z "${NOTIMESTAMP}" ]
then
  INITIAL_COLLECT=0
  if [ -z "${LAST_TIME_BEFORE}" ]
  then
    debug "Initial collect."
    INITIAL_COLLECT=1
    COND1=`echo "(LOCALTIMESTAMP-${COL_TIME}) < INTERVAL '2' DAY"`
    LAST_TIME_BEFORE=`gobackintime`
    store_state_information
  else
    debug "Non-initial collect, start timestamp: '${LAST_TIME_BEFORE}'."
    COND1=`echo "${COL_TIME} >= to_date('${LAST_TIME_BEFORE}','YYYY-MM-DD HH24:MI:SS')"`
  fi


  ######################################################################
  # Obtain the greatest timestamp among all audit records.             #
  ######################################################################
  debug "Obtain the greatest timestamp among all audit records..."
  QUERY=`echo "select to_char(max(${COL_TIME}),'YYYY-MM-DD HH24:MI:SS') from ${TABLE_NAME}"`
  SQLPLUS_RETURN=`${ORACLE_HOME}/bin/sqlplus -S ${ORA_USER}/${ORA_PWD}${ORA_INSTANCE} << ! 2>>$CEA_GENACT_DEBUG_LOG
  set feedback off;
  set echo off;
  set head off;
  set pagesize 0;
  ${QUERY};
  exit;
  !`
  test_sqlplus_code

  LAST_TIME_NOW="${SQLPLUS_RETURN}"
  debug "The greatest timestamp: ${LAST_TIME_NOW}."

  ######################################################################
  # Obtain actual audit records.                                       #
  ######################################################################
  if [ -z "LAST_TIME_NOW" ]
  then
    debug "The greatest timestamp is empty."
  else
    debug "Obtain actual audit records..."
    COND2=`echo "${COL_TIME} < to_date('${LAST_TIME_NOW}','YYYY-MM-DD HH24:MI:SS')"`

    TEST_QUERY2=`echo "select count(*) from ${TABLE_NAME} where ${COND1} and ${COND2}"`
    SQLPLUS_RETURN=`${ORACLE_HOME}/bin/sqlplus -S ${ORA_USER}/${ORA_PWD}${ORA_INSTANCE} << ! 2>>$CEA_GENACT_DEBUG_LOG
      set feedback off;
      set echo off;
      set head off;
      set pagesize 0;
      ${TEST_QUERY2};
      exit;
    !`
    test_sqlplus_code

    RECORD_COUNT_EXP="${SQLPLUS_RETURN}"
    if [ -z "${RECORD_COUNT_EXP}" ]
    then 
      RECORD_COUNT_EXP=-2
      debug "Error occurred when records amount obtaining."
    else
      debug "${RECORD_COUNT_EXP} actual audit records to be collected."
    fi
    
    if [ ! -z "${GETHEADER}" ]
    then
      RECORD_COUNT_EXP=`expr ${RECORD_COUNT_EXP} + 1`
    else
      if [ ! -z "${TABLE_HEAD}" ]
      then
        RECORD_COUNT_EXP=`expr ${RECORD_COUNT_EXP} + 1`
      fi
    fi

    if [ ! -z "${GETHEADER}" ]
    then
      QUERYHEADER=`echo "select ${TABLE_DESC} from ${TABLE_NAME} where rownum=1 order by ${COL_TIME}"`
      SQLPLUS_RETURN=`${ORACLE_HOME}/bin/sqlplus -S ${ORA_USER}/${ORA_PWD}${ORA_INSTANCE} << ! >> /dev/null
      set newpage 0;
      set feedback off;
      set echo off;
      ${PARAM_OUTPUT}
      set linesize ${LINE_SIZE};
      set trimout on;
      set trimspool on;
      ${PARAM_HEADER}
      ${QUERYHEADER};
      exit;
      spool off;
      !`
      if [ ! $? -eq 0 ]; then our_exit 2 "Headline receiving: Sqlplus returned ($?) an error."; else debug "Headline receiving: (OK)"; fi
      
#      HEADER_STRING=`head -n 1 "${SPOOLFILE}" | cut -c 2- | tr -d ' '`  ## doesn't work on AIX - truncate the headline
      HEADER_STRING=`head -n 1 "${SPOOLFILE}"`
      HEADER_STRING=`echo "${HEADER_STRING}" | tr -d ' '`
      HEADER_STRING=`echo "${HEADER_STRING}" | cut -c 2-`
      PARAM_HEADER="${PARAM_HEADER}
      select replace('#Fields: ${HEADER_STRING}',CHR(10),'') from dual;"
    fi
    
    QUERY=`echo "select ${TABLE_DESC} from ${TABLE_NAME} where ${COND1} and ${COND2} order by ${COL_TIME}"`
    SQLPLUS_RETURN=`${ORACLE_HOME}/bin/sqlplus -S ${ORA_USER}/${ORA_PWD}${ORA_INSTANCE} << ! >> /dev/null
    set newpage 0;
    set feedback off;
    set pages 0;
    set echo off;
    ${PARAM_OUTPUT}
    set linesize ${LINE_SIZE};
    set trimout on;
    set trimspool on;
    ${PARAM_HEADER}
    ${QUERY};
    exit;
    spool off;
    !`
    if [ ! $? -eq 0 ]; then our_exit 2 "Collect data receiving: Sqlplus returned ($?) an error."; else debug "Collect data receiving: (OK)"; fi

    mv -f "$SPOOLFILE" "$TEMPFILE" 
    
    RECORD_COUNT_ACT=`wc -l ${TEMPFILE} | sed -e 's/^ *\([0-9]\{1,\}\).*/\1/'`
    if [ ! ${RECORD_COUNT_ACT} -eq ${RECORD_COUNT_EXP} ]
    then
      debug "Records by actuator RECORD_COUNT_ACT=${RECORD_COUNT_ACT};"
      debug "Records by SQLquery RECORD_COUNT_EXP=${RECORD_COUNT_EXP};"
      if [ -z "${NOLINESCHECK}" ]
      then
#        mv -f "$TEMPFILE" "$TEMPFILE"_err
        our_exit 3 "Audit record extraction was a fail, check disk space."
      else
        debug "WARNING: Audit record extraction was a fail, check disk space."
      fi
    fi
  fi
else
  # if timestamp is NOT used
  debug "Obtain records..."

  TEST_QUERY2=`echo "select count(*) from ${TABLE_NAME} "`
  SQLPLUS_RETURN=`${ORACLE_HOME}/bin/sqlplus -S ${ORA_USER}/${ORA_PWD}${ORA_INSTANCE} << ! 2>>$CEA_GENACT_DEBUG_LOG
    set feedback off;
    set echo off;
    set head off;
    set pagesize 0;
    ${TEST_QUERY2};
    exit;
  !`
  test_sqlplus_code

  RECORD_COUNT_EXP=${SQLPLUS_RETURN}
  debug "${RECORD_COUNT_EXP} actual records to be collected."
  if [ ! -z "${GETHEADER}" ]
  then
    RECORD_COUNT_EXP=`expr ${RECORD_COUNT_EXP} + 1`
  else
    if [ ! -z "${TABLE_HEAD}" ]
    then
      RECORD_COUNT_EXP=`expr ${RECORD_COUNT_EXP} + 1`
    fi
  fi

  if [ ! -z "${GETHEADER}" ]
  then
    QUERYHEADER=`echo "select ${TABLE_DESC} from ${TABLE_NAME} where rownum=1"`
    SQLPLUS_RETURN=`${ORACLE_HOME}/bin/sqlplus -S ${ORA_USER}/${ORA_PWD}${ORA_INSTANCE} << ! >> /dev/null
      set newpage 0;
      set feedback off;
      set echo off;
      ${PARAM_OUTPUT}
      set linesize ${LINE_SIZE};
      set trimout on;
      set trimspool on;
      ${PARAM_HEADER}
      ${QUERYHEADER};
    exit;
    spool off;
    !`
    if [ ! $? -eq 0 ]; then our_exit 2 "Headline receiving: Sqlplus returned ($?) an error."; else debug "Headline receiving: (OK)"; fi
#    HEADER_STRING=`head -n 1 "${SPOOLFILE}" | cut -c 2- | tr -d ' '`
    HEADER_STRING=`head -n 1 "${SPOOLFILE}"`
    HEADER_STRING=`echo "${HEADER_STRING}" | tr -d ' '`
    HEADER_STRING=`echo "${HEADER_STRING}" | cut -c 2-`
    PARAM_HEADER="${PARAM_HEADER}
    select replace('#Fields: ${HEADER_STRING}',CHR(10),'') from dual;"
  fi
  
  QUERY=`echo "select ${TABLE_DESC} from ${TABLE_NAME} "`
  SQLPLUS_RETURN=`${ORACLE_HOME}/bin/sqlplus -S ${ORA_USER}/${ORA_PWD}${ORA_INSTANCE} << ! >> /dev/null
    set newpage 0;
    set feedback off;
    set echo off;
    ${PARAM_OUTPUT}
    set linesize ${LINE_SIZE};
    set trimout on;
    set trimspool on;
    ${PARAM_HEADER}
    ${QUERY};
  exit;
  spool off;
  !`
  if [ ! $? -eq 0 ]; then our_exit 2 "Collect data receiving: Sqlplus returned ($?) an error."; else debug "Collect data receiving: (OK)"; fi

  mv -f "$SPOOLFILE" "$TEMPFILE" 

  RECORD_COUNT_ACT=`wc -l ${TEMPFILE} | sed -e 's/^ *\([0-9]\{1,\}\).*/\1/'`
  if [ ! ${RECORD_COUNT_ACT} -eq ${RECORD_COUNT_EXP} ]
  then
    debug "Records by actuator RECORD_COUNT_ACT=${RECORD_COUNT_ACT};"
    debug "Records by SQLquery RECORD_COUNT_EXP=${RECORD_COUNT_EXP};"
    if [ -z "${NOLINESCHECK}" ]
    then
#        mv -f "$TEMPFILE" "$TEMPFILE"_err
      our_exit 3 "Audit record extraction was a fail, check disk space."
    else
      debug "WARNING: Audit record extraction was a fail, check disk space."
    fi
  fi
fi

store_state_information

debug "The collect process was a success (return code 0)"
echo >"${PROPSFILE}.ok"
